@@ -319,6 +319,17 @@ test_that("replace_item", {
     replace_item(x, y, keep_extra = TRUE),
     list(A = 9, B = 3, C = 10)
   )
+
+  x <- list(a = 1, b = list(c = "a", d = FALSE, f = list(x = 0, z = 30)))
+  y <- list(a = 3, e = 2, b = list(d = TRUE, f = list(x = 10, y = 20)))
+  expect_identical(
+    replace_item(x, y, keep_extra = TRUE),
+    list(a = 3, b = list(
+      c = "a", d = TRUE,
+      f = list(x = 10, z = 30, y = 20)
+    ), e = 2)
+  )
+
   expect_error(replace_item(x, c(A = 9, C = 10)))
   expect_error(replace_item(c(A = 1, B = 3), c(A = 9, C = 10, A = 80)))
 })
@@ -393,3 +404,13 @@ test_that("combn_vector", {
     c(8, 2, 2, 3)
   )
 })
+
+
+test_that("broadcast_vector", {
+  expect_identical(broadcast_vector(c(1, 2, 3), 5), c(1, 2, 3, 1, 2))
+})
+
+
+test_that("max_depth", {
+  expect_identical(max_depth(list(a = list(b = list(c = 1), d = 2, e = 3))), 3)
+})
